/***************************************************************************
 Copyright (c) 1998-2002 Novell, Inc.  All Rights Reserved.

 THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
 TREATIES.  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE 
 LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
 THAT CONTAINS THIS WORK.

 Pursuant to the SDK License Agreement, Novell hereby grants to 
 Developer a royalty-free, non-exclusive license to include the
 sample code AES.C and derivative binaries in its product.
 Novell grants to Developer worldwide distribution rights to market,
 distribute or sell the sample code AES.C and derivative
 binaries as a component of Developer's product(s).  Novell shall 
 have no obligations to Developer or Developer's customers with 
 respect to this code.

DISCLAIMER:

   Novell, Inc. makes no representations or warranties with respect
to the contents or use of this code, and specifically disclaims any
express or implied warranties of merchantability or fitness for any
particular purpose.  Further, Novell, Inc. reserves the right to revise
this publication and to make changes to its content, at any time,
without obligation to notify any person or entity of such revisions or
changes.

   Further, Novell, Inc. makes no representations or warranties with
respect to any software, and specifically disclaims any express or
implied warranties of merchantability or fitness for any particular
purpose.  Further, Novell, Inc. reserves the right to make changes to
any and all parts of the software, at any time, without obligation to
notify any person or entity of such changes.                          

***************************************************************************/
#ifdef WIN32
#	include <windows.h>
#endif

#include <stdio.h>
#include "npki.h"
#include "pkierr.h"

NWRCODE StoreServerCerts(void)
{
	NWRCODE				ccode = PKI_SUCCESS;
	NPKIContext			myPKI = NPKI_INVALID_CONTEXT;
	unicode	const		*organizationalCADN = NULL;
	nuint8				*cert = NULL;
	nuint32				certSize = 0;
	nuint8				*CAcert = NULL;
	nuint32				CAcertSize = 0;
	nuint32				finished = 0;
	nuint32				numberOfCertsInList = 0;
	
	// user/system infomation	
	unicode				myTree[]	= {'T','E','S','T',0};
	unicode				myUser[]	= {'A','d','m','i','n','.','n','o','v','e','l','l',0};
	char					password[]	= {'t','e','s','t',0};
	char*					startIPAddress = "192.168.0.2";
	unicode				serverDN[] = {'T','e','s','t','5','1','.','n','o','v','e','l','l',0};
	unicode				certName[] = {'E','x','t','e','r','n','a','l',0};


	ccode = NPKICreateContext(&myPKI);
	if (ccode != PKI_SUCCESS)
	{
		goto ERR_EXIT;
	}
	
	// Set the tree name, this is the tree we will make all calls to.
  	ccode = NPKISetTreeName(myPKI, myTree);	 
	if (ccode != PKI_SUCCESS)
	{
		goto ERR_EXIT;
	}

	ccode = NPKIConnectToIPAddress(myPKI, 0, 0, startIPAddress,	NULL,	NULL);

	// Login in
	ccode = NPKIDSLogin(myPKI, myUser, password);
	if (ccode != PKI_SUCCESS)
	{
		goto ERR_EXIT;
	}

	// make sure the certificate list is cleared before using
	NPKICertificateList(myPKI, NULL, 0, PKI_CLEAR_CERTS, NULL);
	
	/*----------------------------------------------------
		Start --- Important --- Important --- Start
	
    Put your own functions here to get all of the certificates
	------------------------------------------------------*/
	{
		FILE		*stream = NULL;
		size_t	size = 0;
		
	   stream = fopen("ExternalCert.cer", "rb");
		if (stream != NULL)
		{
	   	fseek(stream, 0L, SEEK_END);
			size = ftell(stream);
	   	fseek(stream, 0L, SEEK_SET);
			if ((cert = (nuint8 *)malloc(size)) == NULL)
			{
				ccode = PKI_E_INSUFFICIENT_MEMORY;
				goto ERR_EXIT;
			}
		
			certSize = fread(cert, sizeof(char), size, stream);
			fclose(stream);	
		}
		else
		{
			ccode = PKI_E_FILE_OPEN;
			goto ERR_EXIT;
		}
		
	   stream = fopen("CAcert.cer", "rb");
		if (stream != NULL)
		{
	   	fseek(stream, 0L, SEEK_END);
			size = ftell(stream);
	   	fseek(stream, 0L, SEEK_SET);
			if ((CAcert = (nuint8 *)malloc(size)) == NULL)
			{
				ccode = PKI_E_INSUFFICIENT_MEMORY;
				goto ERR_EXIT;
			}
		
			CAcertSize = fread(CAcert, sizeof(char), size, stream);
			fclose(stream);	
		}
		else
		{
			ccode = PKI_E_FILE_OPEN;
			goto ERR_EXIT;
		}
	}

	/*----------------------------------------------------
		End --- Important --- Important --- End
	------------------------------------------------------*/
	
	// add the object cert to the list
	ccode = NPKICertificateList(myPKI, cert, certSize, PKI_ADD_CERT, NULL);
	if (ccode != PKI_SUCCESS)
	{
		goto ERR_EXIT;
	}
	
	// add the CA cert(s) to the list (If there are more than one cert you must them all)
	ccode = NPKICertificateList(myPKI, CAcert, CAcertSize, PKI_ADD_CERT, NULL);
	if (ccode != PKI_SUCCESS)
	{
		goto ERR_EXIT;
	}
	
	ccode = NPKICertificateList(myPKI, NULL, 0, PKI_SORT_LIST, &numberOfCertsInList);
	if (ccode != PKI_SUCCESS)
	{
		goto ERR_EXIT;
	}

	ccode = NPKIStoreServerCertificatesFromCertificateList
	(
		myPKI, 
		serverDN, 
		certName, 
		0, 
		numberOfCertsInList,
		NULL, 
		NULL
	);
	if (ccode != PKI_SUCCESS)
	{
		goto ERR_EXIT;
	}
	
ERR_EXIT:
	if (cert != NULL)
	{
		free(cert);
	}
	
	if (CAcert != NULL)
	{
		free(CAcert);
	}
	
	NPKIDSLogout(myPKI);
	
	if (myPKI != NPKI_INVALID_CONTEXT)
		NPKIFreeContext(myPKI);
	
	return ccode;
}
